package javax.imageio.spi;

import java.util.Iterator;
import java.security.AccessController;
import java.util.HashMap;
import java.security.AccessControlContext;
import java.util.Map;

class SubRegistry
{
    ServiceRegistry registry;
    Class category;
    final PartiallyOrderedSet poset;
    final Map<Class<?>, Object> map;
    final Map<Class<?>, AccessControlContext> accMap;
    
    public SubRegistry(ServiceRegistry registry, Class category) {
        this.poset = new PartiallyOrderedSet();
        this.map = new HashMap();
        this.accMap = new HashMap();
        this.registry = registry;
        this.category = category;
    }
    
    public boolean registerServiceProvider(Object o) {
        Object value = this.map.get(o.getClass());
        int n = value != null;
        if (!(n == 0)) {
            this.deregisterServiceProvider(value);
        }
        this.map.put(o.getClass(), o);
        this.accMap.put(o.getClass(), AccessController.getContext());
        this.poset.add(o);
        if ((o instanceof RegisterableService)) {
            ((RegisterableService)o).onRegistration(this.registry, this.category);
        }
        return n == 0;
    }
    
    public boolean deregisterServiceProvider(Object o) {
        if (o == this.map.get(o.getClass())) {
            this.map.remove(o.getClass());
            this.accMap.remove(o.getClass());
            this.poset.remove(o);
            if ((o instanceof RegisterableService)) {
                ((RegisterableService)o).onDeregistration(this.registry, this.category);
            }
            return true;
        }
        else {
            return false;
        }
    }
    
    public boolean contains(Object o) {
        return this.map.get(o.getClass()) == o;
    }
    
    public boolean setOrdering(Object o, Object o2) {
        return this.poset.setOrdering(o, o2);
    }
    
    public boolean unsetOrdering(Object o, Object o2) {
        return this.poset.unsetOrdering(o, o2);
    }
    
    public Iterator getServiceProviders(boolean b) {
        if (b) {
            return this.poset.iterator();
        }
        return this.map.values().iterator();
    }
    
    public <T> T getServiceProviderByClass(Class<T> clazz) {
        return this.map.get(clazz);
    }
    
    public void clear() {
        Iterator<Object> iterator = this.map.values().iterator();
        while (iterator.hasNext()) {
            Object next = iterator.next();
            iterator.remove();
            if ((next instanceof RegisterableService)) {
                RegisterableService registerableService = (RegisterableService)next;
                AccessControlContext context = (AccessControlContext)this.accMap.get(next.getClass());
                if (context != null || System.getSecurityManager() == null) {
                    AccessController.doPrivileged(()/*  */ -> {
                        var_3_30.onDeregistration(this.registry, this.category);
                        return null;
                    }, context);
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        this.poset.clear();
        this.accMap.clear();
    }
    
    public void finalize() {
        this.clear();
    }
}
